@@ -10,7 +10,7 @@ def test_algo(algo, output_dir, timeout = 100, nan_fraction = 0.1):
     Function for testing algo functionality.
 
     Args:
-        algo: Learner.
+        algo: Algo class.
         output_dir: local directory for saving model.
         timeout: search time in seconds.
         nan_fraction: probability of null values in data.
@@ -18,7 +18,7 @@ def test_algo(algo, output_dir, timeout = 100, nan_fraction = 0.1):
     Returns:
         None.
     """
-    if isinstance(algo, TabularClassificationAutoML) or isinstance(algo, TabularRegressionAutoML):
+    if issubclass(algo, TabularClassificationAutoML) or issubclass(algo, TabularRegressionAutoML):
         return test_tabular(algo, output_dir, timeout, nan_fraction)
     else:
         raise TestError('invalid algo class')
@@ -38,5 +38,4 @@ def test_package(package, output_dir, **kwargs):
     for field in dir(package):
         attr = getattr(package, field)
         if inspect.isclass(attr) and issubclass(attr, Algo):
-            algo = attr()
-            test_algo(algo, output_dir, **kwargs)
+            test_algo(attr, output_dir, **kwargs)
