@@ -9,8 +9,9 @@ import datetime
 
 from .errors import TestError
 from .targets import create_classification_target, create_regression_target
-from ..base.tabular import TabularClassificationAutoML, TabularRegressionAutoML, is_input_metatype
+from ..base.tabular import TabularClassificationAutoML, TabularRegressionAutoML, is_input_metatype, extract_statics
 from ..base.types import Metatype, is_classification_metatype, is_delimiter_metatype, get_delimiter
+from ..base.errors import ValidationError
 from ..metrics.metrics import get_metric
 
 def create_tabular_inputs(input_metatypes, n_rows = 1000, n_cols = 2, seed = 0):
@@ -60,12 +61,12 @@ def load_tabular_data(algo, nan_fraction = 0.1):
 
     n_cols = 3 if is_multioutput else 1
 
-    if isinstance(algo, TabularClassificationAutoML):
+    if issubclass(algo, TabularClassificationAutoML):
         target_metatype = algo.target_metatype()
         categories = ['bad', 'good'] if target_metatype == Metatype.BINARY else ['red', 'green', 'blue', 'yellow']
         y = create_classification_target(X.index, n_cols, categories)
         y_metatypes = [target_metatype for x in range(n_cols)]
-    elif isinstance(algo, TabularRegressionAutoML):
+    elif issubclass(algo, TabularRegressionAutoML):
         y = create_regression_target(X.index, n_cols)
         y_metatypes = [Metatype.NUMERICAL for x in range(n_cols)]
     else:
@@ -74,48 +75,28 @@ def load_tabular_data(algo, nan_fraction = 0.1):
     return (X, y, X_metatypes, y_metatypes)
 
 def test_tabular(algo, output_dir, timeout, nan_fraction):
-    if not (isinstance(algo, TabularClassificationAutoML) or isinstance(algo, TabularRegressionAutoML)):
+    if not (issubclass(algo, TabularClassificationAutoML) or issubclass(algo, TabularRegressionAutoML)):
         raise TypeError('invalid algo class')
 
-    output = dict()
-
-    input_metatypes = algo.input_metatypes()
-    if not isinstance(input_metatypes, set) or not input_metatypes:
-        raise TestError('input_metatypes must be a non-empty set')
-
-    for metatype in input_metatypes:
-        if not is_input_metatype(metatype):
-            raise TestError('invalid input metatype')
-
-    output['input_metatypes'] = sorted(list(input_metatypes), key=lambda x: x.name)
-
-    is_multioutput = algo.is_multioutput()
-    if not isinstance(is_multioutput, bool):
-        raise TestError('is_multioutput should return bool')
-
-    output['is_multioutput'] = is_multioutput
-
-    if isinstance(algo, TabularClassificationAutoML):
-        target_metatype = algo.target_metatype()
-        if not is_classification_metatype(target_metatype):
-            raise TestError('invalid classification metatype')
-
-        output['target_metatype'] = target_metatype
+    statics = extract_statics(algo)
 
+    if issubclass(algo, TabularClassificationAutoML):
         metric = get_metric('accuracy')
     else:
         metric = get_metric('mean_squared_error')
 
     X, y, X_metatypes, y_metatypes = load_tabular_data(algo, nan_fraction)
 
+    model = algo()
+
     t_start = time.time()
-    algo.fit(timeout, metric, X, y, X_metatypes, y_metatypes, X_unlabelled = X)
+    model.fit(timeout, metric, X, y, X_metatypes, y_metatypes, X_unlabelled = X)
     fit_seconds = time.time() - t_start
 
     if fit_seconds > timeout + 10:
         raise TestError('timeout significantly exceeded')
 
-    pred = algo.predict(X)
+    pred = model.predict(X)
     if not isinstance(pred, pd.DataFrame):
         raise TestError('prediction must be DataFrame')
 
@@ -125,38 +106,29 @@ def test_tabular(algo, output_dir, timeout, nan_fraction):
     if list(pred.index) != list(X.index):
         raise TestError('prediction index must equal X index')
 
-    prediction_methods = ['predict']
-
-    if isinstance(algo, TabularClassificationAutoML):
-        for prediction_method in ('predict_proba', 'decision_function'):
-            try:
-                pred_prediction_method = getattr(algo, prediction_method)(X)
+    if issubclass(algo, TabularClassificationAutoML):
+        for prediction_method in statics['prediction_methods']:
+            if prediction_method != 'predict':
+                pred_prediction_method = getattr(model, prediction_method)(X)
                 if not algo.is_multioutput():
                     pred_prediction_method = [pred_prediction_method]
 
                 for ppm, col in zip(pred_prediction_method, y):
                     _check_prediction(prediction_method, ppm, X, y[col], pred[col])
 
-                prediction_methods.append(prediction_method)
-            except NotImplementedError:
-                pass
-
-    output['prediction_methods'] = sorted(prediction_methods)
-
     if output_dir is not None:
         filename = os.path.join(output_dir, str(uuid.uuid4()) + '.mdl')
-        algo.save(filename)
+        model.save(filename)
 
-        algo_loaded = algo.load(filename)
-        pred_loaded = algo_loaded.predict(X)
+        model_loaded = algo.load(filename)
+        pred_loaded = model_loaded.predict(X)
 
         if not pred_loaded.equals(pred):
             raise TestError('loaded model predictions are off')
 
         os.remove(filename)
 
-    print(f'TESTING DONE: {type(algo).__name__}')
-    return output
+    print(f'TESTING DONE: {type(model).__name__}')
 
 def _check_prediction(prediction_method, ppm, X, y, pred):
     if not isinstance(ppm, pd.DataFrame):
@@ -184,3 +156,5 @@ def _check_prediction(prediction_method, ppm, X, y, pred):
 
     if ppm.shape[1] != len(y_unique) or set(ppm.columns) != y_unique:
         raise TestError(f'{prediction_method} must have a column for each class')
+
+
